@@ -46,6 +46,7 @@ import org.apache.flink.streaming.connectors.kafka.internals.KafkaTopicPartition
 import org.apache.flink.streaming.connectors.kafka.partitioner.FlinkFixedPartitioner;
 import org.apache.flink.streaming.connectors.kafka.partitioner.FlinkKafkaPartitioner;
 import org.apache.flink.streaming.connectors.kafka.table.KafkaConnectorOptions.ScanStartupMode;
+import org.apache.flink.streaming.connectors.kafka.testutils.MockPartitionOffsetsRetriever;
 import org.apache.flink.table.api.DataTypes;
 import org.apache.flink.table.api.ValidationException;
 import org.apache.flink.table.catalog.Column;
@@ -85,7 +86,6 @@ import org.junit.jupiter.params.provider.ValueSource;
 import javax.annotation.Nullable;
 
 import java.util.Arrays;
-import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -95,7 +95,6 @@ import java.util.Optional;
 import java.util.Properties;
 import java.util.Set;
 import java.util.function.Consumer;
-import java.util.function.Function;
 import java.util.regex.Pattern;
 
 import static org.apache.flink.core.testutils.FlinkAssertions.anyCauseMatches;
@@ -579,86 +578,6 @@ public class KafkaDynamicTableFactoryTest {
         validator.accept(kafkaSource);
     }
 
-    private interface OffsetsRetriever
-            extends Function<Collection<TopicPartition>, Map<TopicPartition, Long>> {}
-
-    private interface TimestampOffsetsRetriever
-            extends Function<Map<TopicPartition, Long>, Map<TopicPartition, OffsetAndTimestamp>> {}
-
-    private static final class MockPartitionOffsetsRetriever
-            implements OffsetsInitializer.PartitionOffsetsRetriever {
-
-        public static final OffsetsRetriever UNSUPPORTED_RETRIEVAL =
-                partitions -> {
-                    throw new UnsupportedOperationException(
-                            "The method was not supposed to be called");
-                };
-        private final OffsetsRetriever committedOffsets;
-        private final OffsetsRetriever endOffsets;
-        private final OffsetsRetriever beginningOffsets;
-        private final TimestampOffsetsRetriever offsetsForTimes;
-
-        static MockPartitionOffsetsRetriever noInteractions() {
-            return new MockPartitionOffsetsRetriever(
-                    UNSUPPORTED_RETRIEVAL,
-                    UNSUPPORTED_RETRIEVAL,
-                    UNSUPPORTED_RETRIEVAL,
-                    partitions -> {
-                        throw new UnsupportedOperationException(
-                                "The method was not supposed to be called");
-                    });
-        }
-
-        static MockPartitionOffsetsRetriever timestampAndEnd(
-                TimestampOffsetsRetriever retriever, OffsetsRetriever endOffsets) {
-            return new MockPartitionOffsetsRetriever(
-                    UNSUPPORTED_RETRIEVAL, endOffsets, UNSUPPORTED_RETRIEVAL, retriever);
-        }
-
-        public static MockPartitionOffsetsRetriever latest(OffsetsRetriever endOffsets) {
-            return new MockPartitionOffsetsRetriever(
-                    UNSUPPORTED_RETRIEVAL,
-                    endOffsets,
-                    UNSUPPORTED_RETRIEVAL,
-                    partitions -> {
-                        throw new UnsupportedOperationException(
-                                "The method was not supposed to be called");
-                    });
-        }
-
-        private MockPartitionOffsetsRetriever(
-                OffsetsRetriever committedOffsets,
-                OffsetsRetriever endOffsets,
-                OffsetsRetriever beginningOffsets,
-                TimestampOffsetsRetriever offsetsForTimes) {
-            this.committedOffsets = committedOffsets;
-            this.endOffsets = endOffsets;
-            this.beginningOffsets = beginningOffsets;
-            this.offsetsForTimes = offsetsForTimes;
-        }
-
-        @Override
-        public Map<TopicPartition, Long> committedOffsets(Collection<TopicPartition> partitions) {
-            return committedOffsets.apply(partitions);
-        }
-
-        @Override
-        public Map<TopicPartition, Long> endOffsets(Collection<TopicPartition> partitions) {
-            return endOffsets.apply(partitions);
-        }
-
-        @Override
-        public Map<TopicPartition, Long> beginningOffsets(Collection<TopicPartition> partitions) {
-            return beginningOffsets.apply(partitions);
-        }
-
-        @Override
-        public Map<TopicPartition, OffsetAndTimestamp> offsetsForTimes(
-                Map<TopicPartition, Long> timestampsToSearch) {
-            return offsetsForTimes.apply(timestampsToSearch);
-        }
-    }
-
     @Test
     public void testTableSink() {
         final Map<String, String> modifiedOptions =
@@ -1178,6 +1097,82 @@ public class KafkaDynamicTableFactoryTest {
                                 + " guarantee the semantic of primary key.");
     }
 
+    @Test
+    public void testDiscoverPartitionByDefault() {
+        Map<String, String> tableSourceOptions =
+                getModifiedOptions(
+                        getBasicSourceOptions(),
+                        options -> options.remove("scan.topic-partition-discovery.interval"));
+        final KafkaDynamicSource actualSource =
+                (KafkaDynamicSource) createTableSource(SCHEMA, tableSourceOptions);
+        Properties props = new Properties();
+        props.putAll(KAFKA_SOURCE_PROPERTIES);
+        // The default partition discovery interval is 5 minutes
+        props.setProperty("partition.discovery.interval.ms", "300000");
+        final Map<KafkaTopicPartition, Long> specificOffsets = new HashMap<>();
+        specificOffsets.put(new KafkaTopicPartition(TOPIC, PARTITION_0), OFFSET_0);
+        specificOffsets.put(new KafkaTopicPartition(TOPIC, PARTITION_1), OFFSET_1);
+        final DecodingFormat<DeserializationSchema<RowData>> valueDecodingFormat =
+                new DecodingFormatMock(",", true);
+        // Test scan source equals
+        final KafkaDynamicSource expectedKafkaSource =
+                createExpectedScanSource(
+                        SCHEMA_DATA_TYPE,
+                        null,
+                        valueDecodingFormat,
+                        new int[0],
+                        new int[] {0, 1, 2},
+                        null,
+                        Collections.singletonList(TOPIC),
+                        null,
+                        props,
+                        StartupMode.SPECIFIC_OFFSETS,
+                        specificOffsets,
+                        0);
+        assertThat(actualSource).isEqualTo(expectedKafkaSource);
+        ScanTableSource.ScanRuntimeProvider provider =
+                actualSource.getScanRuntimeProvider(ScanRuntimeProviderContext.INSTANCE);
+        assertKafkaSource(provider);
+    }
+
+    @Test
+    public void testDisableDiscoverPartition() {
+        Map<String, String> tableSourceOptions =
+                getModifiedOptions(
+                        getBasicSourceOptions(),
+                        options -> options.put("scan.topic-partition-discovery.interval", "0"));
+        final KafkaDynamicSource actualSource =
+                (KafkaDynamicSource) createTableSource(SCHEMA, tableSourceOptions);
+        Properties props = new Properties();
+        props.putAll(KAFKA_SOURCE_PROPERTIES);
+        // Disable discovery if the partition discovery interval is 0 minutes
+        props.setProperty("partition.discovery.interval.ms", "0");
+        final Map<KafkaTopicPartition, Long> specificOffsets = new HashMap<>();
+        specificOffsets.put(new KafkaTopicPartition(TOPIC, PARTITION_0), OFFSET_0);
+        specificOffsets.put(new KafkaTopicPartition(TOPIC, PARTITION_1), OFFSET_1);
+        final DecodingFormat<DeserializationSchema<RowData>> valueDecodingFormat =
+                new DecodingFormatMock(",", true);
+        // Test scan source equals
+        final KafkaDynamicSource expectedKafkaSource =
+                createExpectedScanSource(
+                        SCHEMA_DATA_TYPE,
+                        null,
+                        valueDecodingFormat,
+                        new int[0],
+                        new int[] {0, 1, 2},
+                        null,
+                        Collections.singletonList(TOPIC),
+                        null,
+                        props,
+                        StartupMode.SPECIFIC_OFFSETS,
+                        specificOffsets,
+                        0);
+        assertThat(actualSource).isEqualTo(expectedKafkaSource);
+        ScanTableSource.ScanRuntimeProvider provider =
+                actualSource.getScanRuntimeProvider(ScanRuntimeProviderContext.INSTANCE);
+        assertKafkaSource(provider);
+    }
+
     // --------------------------------------------------------------------------------------------
     // Utilities
     // --------------------------------------------------------------------------------------------
